@@ -36,6 +36,7 @@ unsigned int get_next_engine_id()
 CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& context, CFileZillaEngine& parent, EngineNotificationHandler& notificationHandler)
 	: event_handler(context.GetEventLoop())
 	, transfer_status_(*this)
+	, opLockManager_(context.GetOpLockManager())
 	, notification_handler_(notificationHandler)
 	, m_engine_id(get_next_engine_id())
 	, m_options(context.GetOptions())
@@ -45,6 +46,7 @@ CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& contex
 	, parent_(parent)
 	, thread_pool_(context.GetThreadPool())
 	, encoding_converter_(context.GetCustomEncodingConverter())
+	, context_(context)
 {
 	{
 		fz::scoped_lock lock(global_mutex_);
@@ -362,7 +364,7 @@ int CFileZillaEnginePrivate::List(CListCommand const& command)
 					}
 					else {
 						if (!avoid) {
-							CDirectoryListingNotification *pNotification = new CDirectoryListingNotification(pListing->path);
+							CDirectoryListingNotification *pNotification = new CDirectoryListingNotification(pListing->path, true);
 							AddNotification(pNotification);
 						}
 						delete pListing;
