@@ -31,13 +31,12 @@ bool IsMisleadingListResponse(std::wstring const& response)
 }
 }
 
-CFtpListOpData::CFtpListOpData(CFtpControlSocket & controlSocket, CServerPath const& path, std::wstring const& subDir, int flags, bool topLevel)
+CFtpListOpData::CFtpListOpData(CFtpControlSocket & controlSocket, CServerPath const& path, std::wstring const& subDir, int flags)
     : COpData(Command::list, L"CFtpListOpData")
     , CFtpOpData(controlSocket)
     , path_(path)
     , subDir_(subDir)
     , flags_(flags)
-	, topLevel_(topLevel)
 {
 	if (path_.GetType() == DEFAULT) {
 		path_.SetType(currentServer_.GetType());
@@ -61,17 +60,18 @@ int CFtpListOpData::Send()
 		bool is_outdated = false;
 		bool found = engine_.GetDirectoryCache().Lookup(listing, currentServer_, currentPath_, false, is_outdated);
 		if (found && !is_outdated &&
-			(!refresh_ || (holdsLock_ && listing.m_firstListTime >= time_before_locking_)))
+			(!refresh_ || (opLock_ && listing.m_firstListTime >= time_before_locking_)))
 		{
-			controlSocket_.SendDirectoryListingNotification(currentPath_, topLevel_, false);
+			controlSocket_.SendDirectoryListingNotification(currentPath_, false);
 			return FZ_REPLY_OK;
 		}
 
-		if (!holdsLock_) {
-			if (!controlSocket_.TryLock(locking_reason::list, currentPath_)) {
-				time_before_locking_ = fz::monotonic_clock::now();
-				return FZ_REPLY_WOULDBLOCK;
-			}
+		if (!opLock_) {
+			opLock_ = controlSocket_.Lock(locking_reason::list, currentPath_);
+			time_before_locking_ = fz::monotonic_clock::now();
+		}
+		if (opLock_.waiting()) {
+			return FZ_REPLY_WOULDBLOCK;
 		}
 
 		controlSocket_.m_pTransferSocket.reset();
@@ -180,7 +180,7 @@ int CFtpListOpData::ParseResponse()
 
 	engine_.GetDirectoryCache().Store(directoryListing_, currentServer_);
 
-	controlSocket_.SendDirectoryListingNotification(currentPath_, topLevel_, false);
+	controlSocket_.SendDirectoryListingNotification(currentPath_, false);
 
 	return FZ_REPLY_OK;
 }
@@ -254,7 +254,7 @@ int CFtpListOpData::SubcommandResult(int prevResult, COpData const&)
 
 			engine_.GetDirectoryCache().Store(listing, currentServer_);
 
-			controlSocket_.SendDirectoryListingNotification(currentPath_, topLevel_, false);
+			controlSocket_.SendDirectoryListingNotification(currentPath_, false);
 
 			return FZ_REPLY_OK;
 		}
@@ -302,7 +302,7 @@ int CFtpListOpData::SubcommandResult(int prevResult, COpData const&)
 
 				engine_.GetDirectoryCache().Store(listing, currentServer_);
 
-				controlSocket_.SendDirectoryListingNotification(currentPath_, topLevel_, false);
+				controlSocket_.SendDirectoryListingNotification(currentPath_, false);
 
 				return FZ_REPLY_OK;
 			}
@@ -323,14 +323,14 @@ int CFtpListOpData::SubcommandResult(int prevResult, COpData const&)
 
 						engine_.GetDirectoryCache().Store(directoryListing_, currentServer_);
 
-						controlSocket_.SendDirectoryListingNotification(currentPath_, topLevel_, false);
+						controlSocket_.SendDirectoryListingNotification(currentPath_, false);
 
 						return FZ_REPLY_OK;
 					}
 				}
 
 				if (prevResult & FZ_REPLY_ERROR) {
-					controlSocket_.SendDirectoryListingNotification(currentPath_, topLevel_, true);
+					controlSocket_.SendDirectoryListingNotification(currentPath_, true);
 				}
 			}
 
