@@ -4,10 +4,11 @@
 #include "ftp/ftpcontrolsocket.h"
 #include "iothread.h"
 #include "optionsbase.h"
-#include "tlssocket.h"
-#include "transfersocket.h"
 #include "proxy.h"
 #include "servercapabilities.h"
+#include "socket_errors.h"
+#include "tlssocket.h"
+#include "transfersocket.h"
 
 #include <libfilezilla/util.hpp>
 
@@ -32,7 +33,7 @@ CTransferSocket::~CTransferSocket()
 			if (m_transferMode == TransferMode::download) {
 				FinalizeWrite();
 			}
-			ioThread_->SetEventHandler(0);
+			ioThread_->SetEventHandler(nullptr);
 		}
 	}
 }
@@ -41,15 +42,15 @@ void CTransferSocket::ResetSocket()
 {
 	delete m_pProxyBackend;
 	if (m_pBackend == m_pTlsSocket) {
-		m_pBackend = 0;
+		m_pBackend = nullptr;
 	}
 	delete m_pTlsSocket;
 	delete m_pBackend;
 	socketServer_.reset();
 	socket_.reset();
-	m_pProxyBackend = 0;
-	m_pTlsSocket = 0;
-	m_pBackend = 0;
+	m_pProxyBackend = nullptr;
+	m_pTlsSocket = nullptr;
+	m_pBackend = nullptr;
 }
 
 std::wstring CTransferSocket::SetupActiveTransfer(std::string const& ip)
@@ -63,11 +64,11 @@ std::wstring CTransferSocket::SetupActiveTransfer(std::string const& ip)
 	}
 
 	int error;
-	int port = socketServer_->GetLocalPort(error);
+	int port = socketServer_->local_port(error);
 	if (port == -1)	{
 		ResetSocket();
 
-		controlSocket_.LogMessage(MessageType::Debug_Warning, L"GetLocalPort failed: %s", CSocket::GetErrorDescription(error));
+		controlSocket_.LogMessage(MessageType::Debug_Warning, L"GetLocalPort failed: %s", fz::socket_error_description(error));
 		return std::wstring();
 	}
 
@@ -80,7 +81,7 @@ std::wstring CTransferSocket::SetupActiveTransfer(std::string const& ip)
 	}
 
 	std::wstring portArguments;
-	if (socketServer_->GetAddressFamily() == CSocket::ipv6) {
+	if (socketServer_->address_family() == fz::address_type::ipv6) {
 		portArguments = fz::sprintf(L"|2|%s|%d|", ip, port);
 	}
 	else {
@@ -92,27 +93,27 @@ std::wstring CTransferSocket::SetupActiveTransfer(std::string const& ip)
 	return portArguments;
 }
 
-void CTransferSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
+void CTransferSocket::OnSocketEvent(fz::socket_event_source*, fz::socket_event_flag t, int error)
 {
 	if (m_pProxyBackend) {
 		switch (t)
 		{
-		case SocketEventType::connection:
+		case fz::socket_event_flag::connection:
 			{
 				if (error) {
-					controlSocket_.LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), CSocket::GetErrorDescription(error));
+					controlSocket_.LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), fz::socket_error_description(error));
 					TransferEnd(TransferEndReason::failure);
 				}
 				else {
 					delete m_pProxyBackend;
-					m_pProxyBackend = 0;
+					m_pProxyBackend = nullptr;
 					OnConnect();
 				}
 			}
 			return;
-		case SocketEventType::close:
+		case fz::socket_event_flag::close:
 			{
-				controlSocket_.LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), CSocket::GetErrorDescription(error));
+				controlSocket_.LogMessage(MessageType::Error, _("Proxy handshake failed: %s"), fz::socket_error_description(error));
 				TransferEnd(TransferEndReason::failure);
 			}
 			return;
@@ -124,7 +125,7 @@ void CTransferSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int
 	}
 
 	if (socketServer_) {
-		if (t == SocketEventType::connection) {
+		if (t == fz::socket_event_flag::connection) {
 			OnAccept(error);
 		}
 		else {
@@ -135,23 +136,23 @@ void CTransferSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int
 
 	switch (t)
 	{
-	case SocketEventType::connection:
+	case fz::socket_event_flag::connection:
 		if (error) {
 			if (m_transferEndReason == TransferEndReason::none) {
-				controlSocket_.LogMessage(MessageType::Error, _("The data connection could not be established: %s"), CSocket::GetErrorDescription(error));
+				controlSocket_.LogMessage(MessageType::Error, _("The data connection could not be established: %s"), fz::socket_error_description(error));
 				TransferEnd(TransferEndReason::transfer_failure);
 			}
 		}
 		else
 			OnConnect();
 		break;
-	case SocketEventType::read:
+	case fz::socket_event_flag::read:
 		OnReceive();
 		break;
-	case SocketEventType::write:
+	case fz::socket_event_flag::write:
 		OnSend();
 		break;
-	case SocketEventType::close:
+	case fz::socket_event_flag::close:
 		OnClose(error);
 		break;
 	default:
@@ -170,13 +171,13 @@ void CTransferSocket::OnAccept(int error)
 		return;
 	}
 
-	socket_.reset(socketServer_->Accept(error));
+	socket_.reset(socketServer_->accept(error));
 	if (!socket_) {
 		if (error == EAGAIN) {
 			controlSocket_.LogMessage(MessageType::Debug_Verbose, L"No pending connection");
 		}
 		else {
-			controlSocket_.LogMessage(MessageType::Status, _("Could not accept connection: %s"), CSocket::GetErrorDescription(error));
+			controlSocket_.LogMessage(MessageType::Status, _("Could not accept connection: %s"), fz::socket_error_description(error));
 			TransferEnd(TransferEndReason::transfer_failure);
 		}
 		return;
@@ -204,7 +205,7 @@ void CTransferSocket::OnConnect()
 	}
 	else if (m_pTlsSocket) {
 		// Re-enable Nagle algorithm
-		socket_->SetFlags(socket_->GetFlags() & (~CSocket::flag_nodelay));
+		socket_->set_flags(socket_->flags() & (~fz::socket::flag_nodelay));
 		if (CServerCapabilities::GetCapability(controlSocket_.currentServer_, tls_resume) == unknown)	{
 			CServerCapabilities::SetCapability(controlSocket_.currentServer_, tls_resume, m_pTlsSocket->ResumedSession() ? yes : no);
 		}
@@ -239,7 +240,7 @@ void CTransferSocket::OnReceive()
 			if (numread < 0) {
 				delete [] pBuffer;
 				if (error != EAGAIN) {
-					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", CSocket::GetErrorDescription(error));
+					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
 					TransferEnd(TransferEndReason::transfer_failure);
 				}
 				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
@@ -298,7 +299,7 @@ void CTransferSocket::OnReceive()
 
 		if (numread < 0) {
 			if (error != EAGAIN) {
-				controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", CSocket::GetErrorDescription(error));
+				controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
 				TransferEnd(TransferEndReason::transfer_failure);
 			}
 			else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
@@ -309,7 +310,7 @@ void CTransferSocket::OnReceive()
 			FinalizeWrite();
 		}
 		else {
-			send_event<CSocketEvent>(m_pBackend, SocketEventType::read, 0);
+			send_event<fz::socket_event>(m_pBackend, fz::socket_event_flag::read, 0);
 		}
 	}
 	else if (m_transferMode == TransferMode::resumetest) {
@@ -319,7 +320,7 @@ void CTransferSocket::OnReceive()
 			int numread = m_pBackend->Read(buffer, 2, error);
 			if (numread < 0) {
 				if (error != EAGAIN) {
-					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", CSocket::GetErrorDescription(error));
+					controlSocket_.LogMessage(MessageType::Error, L"Could not read from transfer socket: %s", fz::socket_error_description(error));
 					TransferEnd(TransferEndReason::transfer_failure);
 				}
 				else if (m_onCloseCalled && !m_pBackend->IsWaiting(CRateLimiter::inbound)) {
@@ -409,12 +410,12 @@ void CTransferSocket::OnSend()
 			}
 		}
 		else {
-			controlSocket_.LogMessage(MessageType::Error, L"Could not write to transfer socket: %s", CSocket::GetErrorDescription(error));
+			controlSocket_.LogMessage(MessageType::Error, L"Could not write to transfer socket: %s", fz::socket_error_description(error));
 			TransferEnd(TransferEndReason::transfer_failure);
 		}
 	}
 	else if (written > 0) {
-		send_event<CSocketEvent>(m_pBackend, SocketEventType::write, 0);
+		send_event<fz::socket_event>(m_pBackend, fz::socket_event_flag::write, 0);
 	}
 }
 
@@ -450,7 +451,7 @@ void CTransferSocket::OnClose(int error)
 	}
 
 	if (error) {
-		controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), CSocket::GetErrorDescription(error));
+		controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), fz::socket_error_description(error));
 		TransferEnd(TransferEndReason::transfer_failure);
 		return;
 	}
@@ -472,7 +473,7 @@ void CTransferSocket::OnClose(int error)
 		return;
 	}
 	else if (numread < 0 && error != EAGAIN) {
-		controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), CSocket::GetErrorDescription(error));
+		controlSocket_.LogMessage(MessageType::Error, _("Transfer connection interrupted: %s"), fz::socket_error_description(error));
 		TransferEnd(TransferEndReason::transfer_failure);
 		return;
 	}
@@ -497,7 +498,7 @@ bool CTransferSocket::SetupPassiveTransfer(std::wstring const& host, int port)
 
 	ResetSocket();
 
-	socket_ = std::make_unique<CSocket>(engine_.GetThreadPool(), this);
+	socket_ = std::make_unique<fz::socket>(engine_.GetThreadPool(), this);
 
 	if (controlSocket_.m_pProxyBackend) {
 		m_pProxyBackend = new CProxySocket(this, socket_.get(), &controlSocket_);
@@ -511,8 +512,8 @@ bool CTransferSocket::SetupPassiveTransfer(std::wstring const& host, int port)
 			return false;
 		}
 		int error;
-		ip = controlSocket_.m_pSocket->GetPeerIP();
-		port = controlSocket_.m_pSocket->GetRemotePort(error);
+		ip = controlSocket_.socket_->peer_ip();
+		port = controlSocket_.socket_->remote_port(error);
 		if (ip.empty() || port < 1) {
 			controlSocket_.LogMessage(MessageType::Debug_Warning, L"Could not get peer address of control connection.");
 			ResetSocket();
@@ -535,12 +536,12 @@ bool CTransferSocket::SetupPassiveTransfer(std::wstring const& host, int port)
 
 	std::string bindAddress;
 	if (m_pProxyBackend) {
-		bindAddress = controlSocket_.m_pSocket->GetLocalIP();
+		bindAddress = controlSocket_.socket_->local_ip();
 		controlSocket_.LogMessage(MessageType::Debug_Info, L"Binding data connection source IP to control connection source IP %s", bindAddress);
 	}
 	else {
-		if (controlSocket_.m_pSocket->GetPeerIP(true) == ip || controlSocket_.m_pSocket->GetPeerIP(false) == ip) {
-			bindAddress = controlSocket_.m_pSocket->GetLocalIP();
+		if (controlSocket_.socket_->peer_ip(true) == ip || controlSocket_.socket_->peer_ip(false) == ip) {
+			bindAddress = controlSocket_.socket_->local_ip();
 			controlSocket_.LogMessage(MessageType::Debug_Info, L"Binding data connection source IP to control connection source IP %s", bindAddress);
 		}
 		else {
@@ -548,7 +549,7 @@ bool CTransferSocket::SetupPassiveTransfer(std::wstring const& host, int port)
 		}
 	}
 
-	int res = socket_->Connect(fz::to_native(ip), port, CSocket::unspec, bindAddress);
+	int res = socket_->connect(fz::to_native(ip), port, fz::address_type::unknown, bindAddress);
 	if (res && res != EINPROGRESS) {
 		ResetSocket();
 		return false;
@@ -573,7 +574,7 @@ void CTransferSocket::SetActive()
 		return;
 	}
 
-	if (socket_->GetState() == CSocket::connected || socket_->GetState() == CSocket::closing) {
+	if (socket_->get_state() == fz::socket::connected || socket_->get_state() == fz::socket::closing) {
 		TriggerPostponedEvents();
 	}
 }
@@ -592,12 +593,12 @@ void CTransferSocket::TransferEnd(TransferEndReason reason)
 	controlSocket_.send_event<TransferEndEvent>();
 }
 
-std::unique_ptr<CSocket> CTransferSocket::CreateSocketServer(int port)
+std::unique_ptr<fz::listen_socket> CTransferSocket::CreateSocketServer(int port)
 {
-	auto socket = std::make_unique<CSocket>(engine_.GetThreadPool(), this);
-	int res = socket->Listen(controlSocket_.m_pSocket->GetAddressFamily(), port);
+	auto socket = std::make_unique<fz::listen_socket>(engine_.GetThreadPool(), this);
+	int res = socket->listen(controlSocket_.socket_->address_family(), port);
 	if (res) {
-		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"Could not listen on port %d: %s", port, CSocket::GetErrorDescription(res));
+		controlSocket_.LogMessage(MessageType::Debug_Verbose, L"Could not listen on port %d: %s", port, fz::socket_error_description(res));
 		socket.reset();
 	}
 	else {
@@ -607,7 +608,7 @@ std::unique_ptr<CSocket> CTransferSocket::CreateSocketServer(int port)
 	return socket;
 }
 
-std::unique_ptr<CSocket> CTransferSocket::CreateSocketServer()
+std::unique_ptr<fz::listen_socket> CTransferSocket::CreateSocketServer()
 {
 	if (!engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS)) {
 		// Ask the systen for a port
@@ -633,11 +634,11 @@ std::unique_ptr<CSocket> CTransferSocket::CreateSocketServer()
 	}
 
 	if (start < low || start > high) {
-		start = fz::random_number(low, high);
+		start = static_cast<decltype(start)>(fz::random_number(low, high));
 		assert(start >= low && start <= high);
 	}
 
-	std::unique_ptr<CSocket> server;
+	std::unique_ptr<fz::listen_socket> server;
 
 	int count = high - low + 1;
 	while (count--) {
@@ -752,15 +753,15 @@ void CTransferSocket::FinalizeWrite()
 
 bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
 {
-	// Disable Nagle algorithm during TlS handshake
-	socket_->SetFlags(socket_->GetFlags() | CSocket::flag_nodelay);
+	// Disable Nagle's algorithm during TLS handshake
+	socket_->set_flags(socket_->flags() | fz::socket::flag_nodelay);
 
 	assert(!m_pBackend);
 	m_pTlsSocket = new CTlsSocket(this, *socket_, &controlSocket_);
 
 	if (!m_pTlsSocket->Init()) {
 		delete m_pTlsSocket;
-		m_pTlsSocket = 0;
+		m_pTlsSocket = nullptr;
 		return false;
 	}
 
@@ -769,7 +770,7 @@ bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
 	int res = m_pTlsSocket->Handshake(pPrimaryTlsSocket, try_resume);
 	if (res && res != FZ_REPLY_WOULDBLOCK) {
 		delete m_pTlsSocket;
-		m_pTlsSocket = 0;
+		m_pTlsSocket = nullptr;
 		return false;
 	}
 
@@ -826,7 +827,7 @@ bool CTransferSocket::InitBackend()
 	return true;
 }
 
-void CTransferSocket::SetSocketBufferSizes(CSocket& socket)
+void CTransferSocket::SetSocketBufferSizes(fz::socket_base& socket)
 {
 	const int size_read = engine_.GetOptions().GetOptionVal(OPTION_SOCKET_BUFFERSIZE_RECV);
 #if FZ_WINDOWS
@@ -834,12 +835,12 @@ void CTransferSocket::SetSocketBufferSizes(CSocket& socket)
 #else
 	const int size_write = engine_.GetOptions().GetOptionVal(OPTION_SOCKET_BUFFERSIZE_SEND);
 #endif
-	socket.SetBufferSizes(size_read, size_write);
+	socket.set_buffer_sizes(size_read, size_write);
 }
 
 void CTransferSocket::operator()(fz::event_base const& ev)
 {
-	fz::dispatch<CSocketEvent, CIOThreadEvent, fz::timer_event>(ev, this,
+	fz::dispatch<fz::socket_event, CIOThreadEvent, fz::timer_event>(ev, this,
 		&CTransferSocket::OnSocketEvent,
 		&CTransferSocket::OnIOThreadEvent,
 		&CTransferSocket::OnTimer);
@@ -847,10 +848,10 @@ void CTransferSocket::operator()(fz::event_base const& ev)
 
 void CTransferSocket::OnTimer(fz::timer_id)
 {
-	if (socket_ && socket_->GetState() == CSocket::connected) {
-		int const ideal_send_buffer = socket_->GetIdealSendBufferSize();
+	if (socket_ && socket_->get_state() == fz::socket::connected) {
+		int const ideal_send_buffer = socket_->ideal_send_buffer_size();
 		if (ideal_send_buffer != -1) {
-			socket_->SetBufferSizes(-1, ideal_send_buffer);
+			socket_->set_buffer_sizes(-1, ideal_send_buffer);
 		}
 	}
 }
