@@ -1,5 +1,5 @@
-#ifndef __RATELIMITER_H__
-#define __RATELIMITER_H__
+#ifndef FILEZILLA_ENGINE_RATELIMITER_HEADER
+#define FILEZILLA_ENGINE_RATELIMITER_HEADER
 
 #include <option_change_event_handler.h>
 
@@ -28,8 +28,8 @@ protected:
 
 	int GetBucketSize() const;
 
-	std::list<CRateLimiterObject*> m_objectList;
-	std::list<CRateLimiterObject*> m_wakeupList[2];
+	std::vector<CRateLimiterObject*> objects_;
+	std::vector<CRateLimiterObject*> wakeupList_[2];
 
 	fz::timer_id m_timer{};
 
@@ -57,7 +57,7 @@ class CRateLimiterObject
 
 public:
 	CRateLimiterObject();
-	virtual ~CRateLimiterObject() {}
+	virtual ~CRateLimiterObject() = default;
 	int64_t GetAvailableBytes(CRateLimiter::rate_direction direction) const { return m_bytesAvailable[direction]; }
 
 	bool IsWaiting(CRateLimiter::rate_direction direction) const;
@@ -73,4 +73,4 @@ private:
 	int64_t m_bytesAvailable[2];
 };
 
-#endif //__RATELIMITER_H__
+#endif
