@@ -3,6 +3,7 @@
 #include "socket_errors.h"
 #include "tlssocket.h"
 #include "tlssocket_impl.h"
+#include "tls_system_trust_store_impl.h"
 #include "ControlSocket.h"
 
 #include <libfilezilla/iputils.hpp>
@@ -169,10 +170,6 @@ bool CTlsSocketImpl::Init()
 		return false;
 	}
 
-	m_defaultVerifyFlags = gnutls_certificate_get_verify_flags(m_certCredentials);
-
-	gnutls_certificate_set_x509_system_trust(m_certCredentials);
-
 	if (!InitSession()) {
 		return false;
 	}
@@ -356,7 +353,7 @@ ssize_t CTlsSocketImpl::PushFunction(const void* data, size_t len)
 	}
 
 	int error;
-	int written = socketBackend_->Write(data, len, error);
+	int written = socketBackend_->Write(data, static_cast<unsigned int>(len), error);
 
 	if (written < 0) {
 		m_canWriteToSocket = false;
@@ -392,7 +389,7 @@ ssize_t CTlsSocketImpl::PullFunction(void* data, size_t len)
 	}
 
 	int error;
-	int read = socketBackend_->Read(data, len, error);
+	int read = socketBackend_->Read(data, static_cast<unsigned int>(len), error);
 	if (read < 0) {
 		m_canReadFromSocket = false;
 		if (error == EAGAIN) {
@@ -701,7 +698,7 @@ int CTlsSocketImpl::Read(void *buffer, unsigned int len, int& error)
 		TriggerEvents();
 
 		error = 0;
-		return min;
+		return static_cast<int>(min);
 	}
 
 	int res = DoCallGnutlsRecordRecv(buffer, len);
@@ -758,14 +755,14 @@ int CTlsSocketImpl::Write(const void *buffer, unsigned int len, int& error)
 	len -= m_writeSkip;
 	buffer = (char*)buffer + m_writeSkip;
 
-	int res = gnutls_record_send(m_session, buffer, len);
+	ssize_t res = gnutls_record_send(m_session, buffer, len);
 
 	while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket)
 		res = gnutls_record_send(m_session, nullptr, 0);
 
 	if (res >= 0) {
 		error = 0;
-		int written = res + m_writeSkip;
+		int written = static_cast<int>(res) + m_writeSkip;
 		m_writeSkip = 0;
 
 		TriggerEvents();
@@ -786,7 +783,7 @@ int CTlsSocketImpl::Write(const void *buffer, unsigned int len, int& error)
 		}
 	}
 	else {
-		Failure(res, false, L"gnutls_record_send");
+		Failure(static_cast<int>(res), false, L"gnutls_record_send");
 		error = m_socket_error;
 		return -1;
 	}
@@ -812,7 +809,7 @@ void CTlsSocketImpl::TriggerEvents()
 void CTlsSocketImpl::CheckResumeFailedReadWrite()
 {
 	if (m_lastWriteFailed) {
-		int res = GNUTLS_E_AGAIN;
+		ssize_t res = GNUTLS_E_AGAIN;
 		while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket) {
 			res = gnutls_record_send(m_session, nullptr, 0);
 		}
@@ -822,11 +819,11 @@ void CTlsSocketImpl::CheckResumeFailedReadWrite()
 		}
 
 		if (res < 0) {
-			Failure(res, true);
+			Failure(static_cast<int>(res), true);
 			return;
 		}
 
-		m_writeSkip += res;
+		m_writeSkip += static_cast<int>(res);
 		m_lastWriteFailed = false;
 		m_canTriggerWrite = true;
 	}
@@ -876,7 +873,7 @@ int CTlsSocketImpl::Peek(void *buffer, unsigned int len, int& error)
 		memcpy(buffer, peekBuffer_.get(), min);
 
 		error = 0;
-		return min;
+		return static_cast<int>(min);
 	}
 
 	int read = Read(peekBuffer_.get(len), len, error);
@@ -1137,7 +1134,7 @@ std::vector<CCertificate::SubjectName> CTlsSocketImpl::GetCertSubjectAltNames(gn
 			}
 		}
 		else if (type_or_error == GNUTLS_SAN_IPADDRESS) {
-			std::wstring ip = fz::to_wstring(fz::socket::address_to_string(san, san_size));
+			std::wstring ip = fz::to_wstring(fz::socket::address_to_string(san, static_cast<int>(san_size)));
 			if (!ip.empty()) {
 				ret.emplace_back(CCertificate::SubjectName{std::move(ip), false});
 			}
@@ -1350,18 +1347,30 @@ int CTlsSocketImpl::VerifyCertificate()
 
 	// First, check system trust
 	if (uses_hostname) {
-		gnutls_certificate_set_verify_flags(m_certCredentials, m_defaultVerifyFlags);
-		unsigned int status = 0;
-		int verifyResult = gnutls_certificate_verify_peers3(m_session, fz::to_utf8(hostname_).c_str(), &status);
-		if (verifyResult < 0) {
-			m_pOwner->LogMessage(MessageType::Debug_Warning, L"gnutls_certificate_verify_peers2 returned %d with status %u", verifyResult, status);
-			m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
-			Failure(0, true);
-			return FZ_REPLY_ERROR;
-		}
 
-		if (!status) {
-			systemTrust = true;
+		auto lease = m_pOwner->GetEngine().GetContext().GetTlsSystemTrustStore().impl_->lease();
+		auto cred = std::get<0>(lease);
+		if (cred) {
+			gnutls_credentials_set(m_session, GNUTLS_CRD_CERTIFICATE, cred);
+			unsigned int status = 0;
+			int verifyResult = gnutls_certificate_verify_peers3(m_session, fz::to_utf8(hostname_).c_str(), &status);
+			gnutls_credentials_set(m_session, GNUTLS_CRD_CERTIFICATE, m_certCredentials);
+			std::get<1>(lease).unlock();
+
+			if (verifyResult < 0) {
+				m_pOwner->LogMessage(MessageType::Debug_Warning, L"gnutls_certificate_verify_peers2 returned %d with status %u", verifyResult, status);
+				m_pOwner->LogMessage(MessageType::Error, _("Failed to verify peer certificate"));
+				Failure(0, true);
+				return FZ_REPLY_ERROR;
+			}
+
+			if (!status) {
+				systemTrust = true;
+			}
+		}
+		else {
+			std::get<1>(lease).unlock();
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"System trust store could not be loaded");
 		}
 	}
 
@@ -1389,7 +1398,7 @@ int CTlsSocketImpl::VerifyCertificate()
 
 		// 2. Also disable time checks. We allow expired/not yet valid certificates, though only
 		// after explicit user confirmation.
-		gnutls_certificate_set_verify_flags(m_certCredentials, m_defaultVerifyFlags | GNUTLS_VERIFY_DISABLE_TIME_CHECKS | GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS);
+		gnutls_certificate_set_verify_flags(m_certCredentials, gnutls_certificate_get_verify_flags(m_certCredentials) | GNUTLS_VERIFY_DISABLE_TIME_CHECKS | GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS);
 
 		unsigned int status = 0;
 		int verifyResult = gnutls_certificate_verify_peers2(m_session, &status);
@@ -1564,7 +1573,7 @@ std::string CTlsSocketImpl::ListTlsCiphers(std::string priority)
 		return list;
 	}
 	else {
-		for (size_t i = 0; ; ++i) {
+		for (unsigned int i = 0; ; ++i) {
 			unsigned int idx;
 			ret = gnutls_priority_get_cipher_suite_index(pcache, i, &idx);
 			if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE) {
@@ -1594,7 +1603,7 @@ std::string CTlsSocketImpl::ListTlsCiphers(std::string priority)
 
 int CTlsSocketImpl::DoCallGnutlsRecordRecv(void* data, size_t len)
 {
-	int res = gnutls_record_recv(m_session, data, len);
+	ssize_t res = gnutls_record_recv(m_session, data, len);
 	while( (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED) && m_canReadFromSocket && !gnutls_record_get_direction(m_session)) {
 		// Spurious EAGAIN. Can happen if GnuTLS gets a partial
 		// record and the socket got closed.
@@ -1606,7 +1615,7 @@ int CTlsSocketImpl::DoCallGnutlsRecordRecv(void* data, size_t len)
 		res = gnutls_record_recv(m_session, data, len);
 	}
 
-	return res;
+	return static_cast<int>(res);
 }
 
 std::wstring CTlsSocketImpl::GetGnutlsVersion()
