@@ -22,47 +22,45 @@ CClearPrivateDataDialog::CClearPrivateDataDialog(CMainFrame* pMainFrame)
 
 void CClearPrivateDataDialog::Run()
 {
-	if (!Load(m_pMainFrame, _T("ID_CLEARPRIVATEDATA")))
+	if (!Load(m_pMainFrame, _T("ID_CLEARPRIVATEDATA"))) {
 		return;
+	}
 
-	if (ShowModal() != wxID_OK)
+	if (ShowModal() != wxID_OK) {
 		return;
+	}
 
 	wxCheckBox *pSitemanagerCheck = XRCCTRL(*this, "ID_CLEARSITEMANAGER", wxCheckBox);
 	wxCheckBox *pQueueCheck = XRCCTRL(*this, "ID_CLEARQUEUE", wxCheckBox);
-	if (pSitemanagerCheck->GetValue() && pQueueCheck->GetValue())
-	{
+	if (pSitemanagerCheck->GetValue() && pQueueCheck->GetValue()) {
 		int res = wxMessageBoxEx(_("Do you really want to delete all Site Manager entries and the transfer queue?"), _("Clear private data"), wxYES | wxNO | wxICON_QUESTION);
-		if (res != wxYES)
+		if (res != wxYES) {
 			return;
+		}
 	}
-	else if (pQueueCheck->GetValue())
-	{
+	else if (pQueueCheck->GetValue()) {
 		int res = wxMessageBoxEx(_("Do you really want to delete the transfer queue?"), _("Clear private data"), wxYES | wxNO | wxICON_QUESTION);
-		if (res != wxYES)
+		if (res != wxYES) {
 			return;
+		}
 	}
-	else if (pSitemanagerCheck->GetValue())
-	{
+	else if (pSitemanagerCheck->GetValue()) {
 		int res = wxMessageBoxEx(_("Do you really want to delete all Site Manager entries?"), _("Clear private data"), wxYES | wxNO | wxICON_QUESTION);
-		if (res != wxYES)
+		if (res != wxYES) {
 			return;
+		}
 	}
 
 	wxCheckBox *pCheck = XRCCTRL(*this, "ID_CLEARQUICKCONNECT", wxCheckBox);
-	if (!pCheck)
-		return;
-
-	if (pCheck->GetValue())
-	{
+	if (pCheck && pCheck->GetValue()) {
 		CRecentServerList::Clear();
-		if (m_pMainFrame->GetQuickconnectBar())
+		if (m_pMainFrame->GetQuickconnectBar()) {
 			m_pMainFrame->GetQuickconnectBar()->ClearFields();
+		}
 	}
 
 	pCheck = XRCCTRL(*this, "ID_CLEARRECONNECT", wxCheckBox);
-
-	if (pCheck->GetValue()) {
+	if (pCheck && pCheck->GetValue()) {
 		bool asked = false;
 
 		const std::vector<CState*> *states = CContextManager::Get()->GetAllStates();
@@ -72,8 +70,9 @@ void CClearPrivateDataDialog::Run()
 			if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
 				if (!asked) {
 					int res = wxMessageBoxEx(_("Reconnect information cannot be cleared while connected to a server.\nIf you continue, your connection will be disconnected."), _("Clear private data"), wxOK | wxCANCEL);
-					if (res != wxOK)
+					if (res != wxOK) {
 						return;
+					}
 					asked = true;
 				}
 
@@ -83,8 +82,9 @@ void CClearPrivateDataDialog::Run()
 					m_timer.SetOwner(this);
 					m_timer.Start(250, true);
 				}
-				else
+				else {
 					pState->Disconnect();
+				}
 			}
 		}
 
@@ -96,8 +96,9 @@ void CClearPrivateDataDialog::Run()
 		CInterProcessMutex sitemanagerMutex(MUTEX_SITEMANAGERGLOBAL, false);
 		while (sitemanagerMutex.TryLock() == 0) {
 			int res = wxMessageBoxEx(_("The Site Manager is opened in another instance of FileZilla 3.\nPlease close it or the data cannot be deleted."), _("Clear private data"), wxOK | wxCANCEL);
-			if (res != wxYES)
+			if (res != wxYES) {
 				return;
+			}
 		}
 		CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 		RemoveXmlFile(_T("sitemanager"));
@@ -116,21 +117,20 @@ void CClearPrivateDataDialog::OnTimer(wxTimerEvent&)
 {
 	const std::vector<CState*> *states = CContextManager::Get()->GetAllStates();
 
-	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter)
-	{
+	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter) {
 		CState* pState = *iter;
 
-		if (pState->IsRemoteConnected() || !pState->IsRemoteIdle())
-		{
-			if (!pState->m_pCommandQueue->Cancel())
+		if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
+			if (!pState->m_pCommandQueue->Cancel()) {
 				return;
+			}
 
 			pState->Disconnect();
 		}
 
-		if (pState->IsRemoteConnected() || !pState->IsRemoteIdle())
+		if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
 			return;
-
+		}
 	}
 
 	m_timer.Stop();
@@ -140,23 +140,25 @@ void CClearPrivateDataDialog::OnTimer(wxTimerEvent&)
 
 void CClearPrivateDataDialog::Delete()
 {
-	if (m_timer.IsRunning())
+	if (m_timer.IsRunning()) {
 		return;
+	}
 
 	Destroy();
 }
 
 bool CClearPrivateDataDialog::ClearReconnect()
 {
-	COptions::Get()->SetLastServer(CServer());
-	COptions::Get()->SetOption(OPTION_LASTSERVERPATH, _T(""));
+	COptions::Get()->SetOptionXml(OPTION_TAB_DATA, pugi::xml_node());
+	COptions::Get()->RequireCleanup();
+	COptions::Get()->SaveIfNeeded();
 
 	const std::vector<CState*> *states = CContextManager::Get()->GetAllStates();
-	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter)
-	{
+	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter) {
 		CState* pState = *iter;
-
-		pState->SetLastServer(CServer(), CServerPath());
+		if (pState) {
+			pState->SetLastSite(Site(), CServerPath());
+		}
 	}
 
 	return true;
@@ -166,12 +168,14 @@ void CClearPrivateDataDialog::RemoveXmlFile(const wxString& name)
 {
 	{
 		wxFileName fn(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), name + _T(".xml"));
-		if (fn.FileExists())
+		if (fn.FileExists()) {
 			wxRemoveFile(fn.GetFullPath());
+		}
 	}
 	{
-		wxFileName fn(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), name + _T("xml~"));
-		if (fn.FileExists())
+		wxFileName fn(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), name + _T(".xml~"));
+		if (fn.FileExists()) {
 			wxRemoveFile(fn.GetFullPath());
+		}
 	}
 }
