@@ -1,21 +1,23 @@
-#ifndef __CLEARPRIVATEDATA_H__
-#define __CLEARPRIVATEDATA_H__
+#ifndef FILEZILLA_INTERFACE_CLEARPRIVATEDATA_HEADER
+#define FILEZILLA_INTERFACE_CLEARPRIVATEDATA_HEADER
 
 #include "dialogex.h"
 
+#include <wx/timer.h>
+
 class CMainFrame;
-class CClearPrivateDataDialog : public wxDialogEx
+class CClearPrivateDataDialog final : public wxDialogEx
 {
 public:
 
 	static CClearPrivateDataDialog* Create(CMainFrame* pMainFrame) { return new CClearPrivateDataDialog(pMainFrame); }
-	void Show();
+	void Run();
 
 	void Delete();
 
 protected:
 	CClearPrivateDataDialog(CMainFrame* pMainFrame);
-	~CClearPrivateDataDialog() {}
+	virtual ~CClearPrivateDataDialog() = default;
 
 	bool ClearReconnect();
 
@@ -29,5 +31,4 @@ protected:
 	void OnTimer(wxTimerEvent& event);
 };
 
-#endif //__CLEARPRIVATEDATA_H__
-
+#endif
